#!/bin/bash -l

#SBATCH -J ONT_GDB
#SBATCH -N 1
#SBATCH -n 1
#SBATCH -c 15
#SBATCH --time=0-01:00:00
#SBATCH -p batch
#SBATCH --qos=qos-batch

# NOTE: script to compute and plot metaT and metaG coverage for a specific assembly
#       and genes not found in another assembly
# NOTE: this is for testing purposes only !!!

# Looking for conda env.s
#   grep -r --include="*.yaml" "samtools" ~/miniconda3/envs/ONT_pilot/pipeline/
# Contig cov. and length:
#   sort -n -k2,2 ../results/mapping/metat/${rtype2}/${tool2}/ASSEMBLY.FILTERED.sr.cov.ave.txt | tail -n 10
#   sort -n -k2,2 ../results/mapping/metat/${rtype2}/${tool2}/ASSEMBLY.FILTERED.sr.cov.ave.txt | tail -n 10 | cut -f1 -d" " | while read f; do grep "${f}" ../results/mapping/metat/${rtype2}/${tool2}/ASSEMBLY.FILTERED.sr.idxstats.txt; done

# TEST: lr_flye, contig_446, sr_metaspades
# TEST: lr_flye_sr, contig_446, sr_metaspades

cd /mnt/lscratch/users/vgalata/ont_pilot/cdhit_cov_example

rtype1="sr"; tool1="metaspades"
rtype2="lr"; tool2="flye_sr"

contig="contig_446"

threads=15
sr1="../results/preproc/metag/sr/R1.fastp.fastq.gz"
sr2="../results/preproc/metag/sr/R2.fastp.fastq.gz"
asm="../results/assembly/${rtype2}/${tool2}/ASSEMBLY.FILTERED.fasta"
idx="../results/assembly/${rtype2}/${tool2}/ASSEMBLY.FILTERED"
faa="../results/annotation/prodigal/${rtype2}/${tool2}/proteins.faa"
uniq="../results/analysis/cdhit/${rtype1}_${tool1}__${rtype2}_${tool2}.faa"

bam_metaG="${rtype2}_${tool2}.metaG.bam"
bam_metaT="../results/mapping/metat/${rtype2}/${tool2}/ASSEMBLY.FILTERED.sr.bam"
cov_metaG="${rtype2}_${tool2}.metaG.cov"
cov_metaT="${rtype2}_${tool2}.metaT.cov"
ids_all="${rtype2}_${tool2}_${contig}.ids"
ids_uniq="${rtype2}_${tool2}_${contig}.uniq.ids"
pdf="${rtype2}_${tool2}_${contig}.pdf"

# BAMs
# metaG, SR: as in the rules
# metaT, SR: already exists
conda activate /home/users/vgalata/miniconda3/envs/ONT_pilot/pipeline/7070f59f
bwa mem -t ${threads} ${idx} ${sr1} ${sr2} | samtools view -@ ${threads} -SbT ${asm} | samtools sort -@ ${threads} -m 4G -T ${bam_metaG%.*} > ${bam_metaG}
conda deactivate

# genome coverage in required format
# https://bedtools.readthedocs.io/en/latest/content/tools/genomecov.html
conda activate /scratch/users/vgalata/miniconda3/ONT_pilot/pipeline/121c497d
bedtools genomecov -bg -ibam ${bam_metaG} > ${cov_metaG}
bedtools genomecov -bg -ibam ${bam_metaT} > ${cov_metaT}
conda deactivate
awk -F"\t" '$1=="'${contig}'" {print $0}' ${cov_metaG} > ${cov_metaG}.${contig}
awk -F"\t" '$1=="'${contig}'" {print $0}' ${cov_metaT} > ${cov_metaT}.${contig}

# all/uniq genes from Flye
grep "${contig}_" ${faa} | sed "s/^>${contig}_//;s/ # /\t/g" > ${ids_all}
grep "^>${rtype2}__${tool2}__${contig}_" ${uniq} | sed "s/^>${rtype2}__${tool2}__${contig}_//;s/ # /\t/g" | sort -n -k1,1 > ${ids_uniq}

# plot
conda activate /home/users/vgalata/miniconda3/envs/ONT_pilot/pipeline/e231f1b8
Rscript /mnt/irisgpfs/users/vgalata/projects/ONT_pilot/notes/cdhit_uniq_cov.R ${cov_metaG}.${contig} ${cov_metaT}.${contig} ${ids_uniq} ${ids_all} "Proteins in ${rtype2}_${tool2} but not in ${rtype1}_${tool1}, contig ${contig}" ${pdf}
conda deactivate