# Mapping (to assembly)

##################################################
# For polishing

# rule mapping_bwa_idx_polishing:
#     input:
#         os.path.join(RESULTS_DIR, "assembly/lr/{tool}/assembly.fasta")
#     output:
#         temp(expand(os.path.join(RESULTS_DIR, "assembly/lr/{{tool}}/polishing/assembly.{ext}"), ext=BWA_IDX_EXT))
#     log:
#         "logs/bwa_index.polishing.{tool}.log"
#     wildcard_constraints:
#         tool="|".join(config["assemblers"]["lr"])
#     threads: 1
#     params:
#         idx_prefix=lambda wildcards, output: os.path.splitext(output[0])[0]
#     conda:
#         os.path.join(ENV_DIR, "racon.yaml")
#     message:
#         "Mapping: BWA index for assembly polishing"
#     shell:
#         "(date && bwa index {input} -p {params.idx_prefix} && date) &> {log}"

# rule mapping_bwa_mem_polishing:
#     input:
#         lr=os.path.join(RESULTS_DIR, "preproc/metag/lr/lr.fastq.gz"),
#         asm=os.path.join(RESULTS_DIR, "assembly/lr/{tool}/assembly.fasta"),
#         idx=expand(os.path.join(RESULTS_DIR, "assembly/lr/{{tool}}/polishing/assembly.{ext}"), ext=BWA_IDX_EXT)
#     output:
#         temp(os.path.join(RESULTS_DIR, "mapping/metag/lr/{tool}/polishing/assembly_lr.bam"))
#     log:
#         "logs/bwa_mem.polishing.metag.lr.{tool}.log"
#     wildcard_constraints:
#         tool="|".join(config["assemblers"]["lr"])
#     threads:
#         config["bwa"]["threads"]
#     params:
#         idx_prefix=lambda wildcards, input: os.path.splitext(input.idx[0])[0],
#         bam_prefix=lambda wildcards, output: os.path.splitext(output[0])[0]
#     conda:
#         os.path.join(ENV_DIR, "racon.yaml")
#     message:
#         "Mapping long reads to assembly w/ BWA for polishing"
#     shell:
#         "(date && "
#         "bwa mem -x ont2d -t {threads} {params.idx_prefix} {input.lr} | "
#         "samtools view -@ {threads} -SbT {input.asm} | "
#         "samtools sort -@ {threads} -m {config[samtools][sort][chunk_size]} -T {params.bam_prefix} > {output} && "
#         "date) &> {log}"

##################################################
# For metaT polishing

# NOTE: for racon
# rule mapping_bwa_mem_polishing_metat:
#     input:
#         # reads
#         r1=os.path.join(RESULTS_DIR, "preproc/metat/sr/R1.fastp.fastq.gz"),
#         r2=os.path.join(RESULTS_DIR, "preproc/metat/sr/R2.fastp.fastq.gz"),
#         # assembly
#         asm=os.path.join(RESULTS_DIR, "assembly/{rtype}/{tool}/ASSEMBLY.fasta"),
#         # assembly index
#         idx=expand(os.path.join(RESULTS_DIR, "assembly/{{rtype}}/{{tool}}/ASSEMBLY.{ext}"), ext=BWA_IDX_EXT)
#     output:
#         sam=temp(os.path.join(RESULTS_DIR, "assembly/{rtype}/{tool}metatracon/mapping.sam")),
#         fastq=temp(os.path.join(RESULTS_DIR, "assembly/{rtype}/{tool}metatracon/reads.fastq"))
#     log:
#         "logs/bwa_mem.polishing.metat.{rtype}.{tool}.log"
#     wildcard_constraints:
#         rtype="|".join(READ_TYPES),
#         tool="|".join(ASSEMBLERS)
#     threads:
#         config["bwa"]["threads"]
#     params:
#         idx_prefix=lambda wildcards, input: os.path.splitext(input.idx[0])[0],
#         sam_prefix=lambda wildcards, output: os.path.splitext(output.sam)[0]
#     conda:
#         os.path.join(ENV_DIR, "racon.yaml")
#     message:
#         "Mapping short reads to assembly w/ BWA"
#     shell:
#         "(date && "
#         "zcat {input.r1} | awk '{{print (NR%4 == 1) ? \"@1_\" ++i : $0}}' > {output.fastq} && "
#         "zcat {input.r2} | awk '{{print (NR%4 == 1) ? \"@2_\" ++i : $0}}' >> {output.fastq} && "
#         "bwa mem -t {threads} {params.idx_prefix} {output.fastq} | "
#         "samtools view -@ {threads} -SbT {input.asm} | "
#         "samtools sort -@ {threads} -m {config[samtools][sort][chunk_size]} -T {params.sam_prefix} | "
#         "samtools view -@ {threads} -h > {output.sam} && "
#         "date) &> {log}"

##################################################
# For assembly

# BWA index
rule mapping_bwa_idx_assembly:
    input:
        os.path.join(RESULTS_DIR, "assembly/{rtype}/{tool}/{atype}.fasta")
    output:
        expand(os.path.join(RESULTS_DIR, "assembly/{{rtype}}/{{tool}}/{{atype}}.{ext}"), ext=BWA_IDX_EXT)
    log:
        "logs/bwa.index.{rtype}.{tool}.{atype}.log"
    wildcard_constraints:
        rtype="|".join(READ_TYPES),
        tool="|".join(ASSEMBLERS),
        atype="ASSEMBLY|ASSEMBLY.FILTERED"
    threads: 1
    params:
        idx_prefix=lambda wildcards, output: os.path.splitext(output[0])[0]
    conda:
        os.path.join(ENV_DIR, "racon.yaml")
    message:
        "Mapping: BWA index for assembly mapping"
    shell:
        "(date && bwa index {input} -p {params.idx_prefix} && date) &> {log}"

# Short reads
rule mapping_bwa_mem_assembly_sr_metag:
    input:
        r1=os.path.join(RESULTS_DIR, "preproc/metag/sr/R1.fastp.fastq.gz"),
        r2=os.path.join(RESULTS_DIR, "preproc/metag/sr/R2.fastp.fastq.gz"),
        asm=os.path.join(RESULTS_DIR, "assembly/{rtype}/{tool}/{atype}.fasta"),
        idx=expand(os.path.join(RESULTS_DIR, "assembly/{{rtype}}/{{tool}}/{{atype}}.{ext}"), ext=BWA_IDX_EXT)
    output:
        os.path.join(RESULTS_DIR, "mapping/metag/{rtype}/{tool}/{atype}.sr.bam")
    log:
        "logs/bwa.mem.metag.{rtype}.{tool}.{atype}.sr.log",
    wildcard_constraints:
        mtype="|".join(META_TYPES),
        rtype="|".join(READ_TYPES),
        tool="|".join(ASSEMBLERS),
        atype="ASSEMBLY|ASSEMBLY.FILTERED"
    threads:
        config["bwa"]["threads"]
    params:
        idx_prefix=lambda wildcards, input: os.path.splitext(input.idx[0])[0],
        bam_prefix=lambda wildcards, output: os.path.splitext(output[0])[0],
        chunk_size=config["samtools"]["sort"]["chunk_size"]
    conda:
        os.path.join(ENV_DIR, "racon.yaml")
    message:
        "Mapping short reads to assembly w/ BWA"
    shell:
        "(date && "
        "bwa mem -t {threads} {params.idx_prefix} {input.r1} {input.r2} | "
        "samtools view -@ {threads} -SbT {input.asm} | "
        "samtools sort -@ {threads} -m {params.chunk_size} -T {params.bam_prefix} > {output} && "
        "date) &> {log}"

rule mapping_bwa_mem_assembly_sr_metat:
    input:
        r1=os.path.join(RESULTS_DIR, "preproc/metat/sr/R1.norrna.fastq.gz"),
        r2=os.path.join(RESULTS_DIR, "preproc/metat/sr/R2.norrna.fastq.gz"),
        asm=os.path.join(RESULTS_DIR, "assembly/{rtype}/{tool}/{atype}.fasta"),
        idx=expand(os.path.join(RESULTS_DIR, "assembly/{{rtype}}/{{tool}}/{{atype}}.{ext}"), ext=BWA_IDX_EXT)
    output:
        os.path.join(RESULTS_DIR, "mapping/metat/{rtype}/{tool}/{atype}.sr.bam")
    log:
        "logs/bwa.mem.metat.{rtype}.{tool}.{atype}.sr.log",
    wildcard_constraints:
        rtype="|".join(READ_TYPES),
        tool="|".join(ASSEMBLERS),
        atype="ASSEMBLY|ASSEMBLY.FILTERED"
    threads:
        config["bwa"]["threads"]
    params:
        idx_prefix=lambda wildcards, input: os.path.splitext(input.idx[0])[0],
        bam_prefix=lambda wildcards, output: os.path.splitext(output[0])[0],
        chunk_size=config["samtools"]["sort"]["chunk_size"]
    conda:
        os.path.join(ENV_DIR, "racon.yaml")
    message:
        "Mapping short reads to assembly w/ BWA"
    shell:
        "(date && "
        "bwa mem -t {threads} {params.idx_prefix} {input.r1} {input.r2} | "
        "samtools view -@ {threads} -SbT {input.asm} | "
        "samtools sort -@ {threads} -m {params.chunk_size} -T {params.bam_prefix} > {output} && "
        "date) &> {log}"

# Long reads
rule mapping_bwa_mem_assembly_lr:
    input:
        lr=os.path.join(RESULTS_DIR, "preproc/metag/lr/lr.fastq.gz"),
        asm=os.path.join(RESULTS_DIR, "assembly/{rtype}/{tool}/{atype}.fasta"),
        idx=expand(os.path.join(RESULTS_DIR, "assembly/{{rtype}}/{{tool}}/{{atype}}.{ext}"), ext=BWA_IDX_EXT)
    output:
        os.path.join(RESULTS_DIR, "mapping/metag/{rtype}/{tool}/{atype}.lr.bam")
    log:
        "logs/bwa.mem.metag.{rtype}.{tool}.{atype}.lr.log",
    wildcard_constraints:
        rtype="lr|hy|hyhy",
        tool="|".join(config["assemblers"]["lr"] + config["assemblers"]["hy"] + config["assemblers"]["hyhy"]),
        atype="ASSEMBLY|ASSEMBLY.FILTERED"
    threads:
        config["bwa"]["threads"]
    params:
        idx_prefix=lambda wildcards, input: os.path.splitext(input.idx[0])[0],
        bam_prefix=lambda wildcards, output: os.path.splitext(output[0])[0],
        chunk_size=config["samtools"]["sort"]["chunk_size"]
    conda:
        os.path.join(ENV_DIR, "racon.yaml")
    message:
        "Mapping long reads to assembly w/ BWA"
    shell:
        "(date && "
        "bwa mem -x ont2d -t {threads} {params.idx_prefix} {input.lr} | "
        "samtools view -@ {threads} -SbT {input.asm} | "
        "samtools sort -@ {threads} -m {params.chunk_size} -T {params.bam_prefix} > {output} && "
        "date) &> {log}"

# Hybrid: short + long reads
rule mapping_bwa_mem_assembly_hy:
    input:
        sr=os.path.join(RESULTS_DIR, "mapping/metag/{rtype}/{tool}/{atype}.sr.bam"),
        lr=os.path.join(RESULTS_DIR, "mapping/metag/{rtype}/{tool}/{atype}.lr.bam")
    output:
        os.path.join(RESULTS_DIR, "mapping/metag/{rtype}/{tool}/{atype}.hy.bam")
    log:
        "logs/bwa_mem.metag.hy.{rtype}.{tool}.{atype}.hy.log"
    wildcard_constraints:
        tool="|".join(config["assemblers"]["hy"]),
        atype="ASSEMBLY|ASSEMBLY.FILTERED"
    threads: 1
    conda:
        os.path.join(ENV_DIR, "racon.yaml")
    message:
        "Mapping: merging short-reads and long-reads mapping results"
    shell:
        "(date && samtools merge {output} {input.sr} {input.lr} && date) &> {log}"

# Hybrid: "short + long reads" hybrid, "metaT and metaG" hybrid
# NOTE: BAM for metaT reads is created in the rule since for other assemblies only filtered reads are used
rule mapping_bwa_mem_assembly_hyhy:
    input:
        bam_sr=os.path.join(RESULTS_DIR, "mapping/metag/hyhy/{tool}/{atype}.sr.bam"),
        bam_lr=os.path.join(RESULTS_DIR, "mapping/metag/hyhy/{tool}/{atype}.lr.bam"),
        metat_r1=os.path.join(RESULTS_DIR, "preproc/metat/sr/R1.fastp.fastq.gz"),
        metat_r2=os.path.join(RESULTS_DIR, "preproc/metat/sr/R2.fastp.fastq.gz"),
        asm=os.path.join(RESULTS_DIR, "assembly/{rtype}/{tool}/{atype}.fasta"),
        idx=expand(os.path.join(RESULTS_DIR, "assembly/{{rtype}}/{{tool}}/{{atype}}.{ext}"), ext=BWA_IDX_EXT)
    output:
        bam_metat=temp(os.path.join(RESULTS_DIR, "mapping/metag/{rtype}/{tool}/{atype}.hyhy.metat.bam")),
        bam=os.path.join(RESULTS_DIR, "mapping/metag/{rtype}/{tool}/{atype}.hyhy.bam")
    log:
        "logs/bwa_mem.metag.hyhy.{rtype}.{tool}.{atype}.hyhy.log"
    wildcard_constraints:
        tool="|".join(config["assemblers"]["hyhy"]),
        atype="ASSEMBLY|ASSEMBLY.FILTERED"
    threads:
        config["bwa"]["threads"]
    params:
        idx_prefix=lambda wildcards, input: os.path.splitext(input.idx[0])[0],
        bam_prefix=lambda wildcards, output: os.path.splitext(output.bam_metat)[0],
        chunk_size=config["samtools"]["sort"]["chunk_size"]
    conda:
        os.path.join(ENV_DIR, "racon.yaml")
    message:
        "Mapping: merging short-reads and long-reads mapping results"
    shell:
        "(date && "
        "bwa mem -t {threads} {params.idx_prefix} {input.metat_r1} {input.metat_r2} | "
        "samtools view -@ {threads} -SbT {input.asm} | "
        "samtools sort -@ {threads} -m {params.chunk_size} -T {params.bam_prefix} > {output.bam_metat} && "
        "samtools merge {output.bam} {input.bam_sr} {output.bam_metat} {input.bam_lr} && "
        "date) &> {log}"

##################################################
# Assembly coverage

# Compute the genome coverage histogram: https://bedtools.readthedocs.io/en/latest/content/tools/genomecov.html
rule mapping_assembly_genomecov:
    input:
        os.path.join(RESULTS_DIR, "mapping/{mtype}/{rtype}/{tool}/{atype}.{rtype2}.bam")
    output:
        os.path.join(RESULTS_DIR, "mapping/{mtype}/{rtype}/{tool}/{atype}.{rtype2}.cov.txt")
    log:
        "logs/bedtools.genomecov.{mtype}.{rtype}.{tool}.{atype}.{rtype2}.log",
    wildcard_constraints:
        mtype="|".join(META_TYPES),
        rtype="|".join(READ_TYPES),
        rtype2="|".join(READ_TYPES),
        tool="|".join(ASSEMBLERS),
        atype="ASSEMBLY|ASSEMBLY.FILTERED"
    threads: 1
    conda:
        os.path.join(ENV_DIR, "bedtools.yaml")
    message:
        "Mapping: compute assembly coverage"
    shell:
        "(date && bedtools genomecov -ibam {input} > {output} && date) &> {log}"

rule mapping_assembly_genomecov_average:
    input:
        os.path.join(RESULTS_DIR, "mapping/{mtype}/{rtype}/{tool}/{atype}.{rtype2}.cov.txt")
    output:
        os.path.join(RESULTS_DIR, "mapping/{mtype}/{rtype}/{tool}/{atype}.{rtype2}.cov.ave.txt")
    log:
        "logs/ave.genomecov.{mtype}.{rtype}.{tool}.{atype}.{rtype2}.log"
    wildcard_constraints:
        mtype="|".join(META_TYPES),
        rtype="|".join(READ_TYPES),
        rtype2="|".join(READ_TYPES),
        tool="|".join(ASSEMBLERS),
        atype="ASSEMBLY|ASSEMBLY.FILTERED"
    threads: 1
    params:
        script=os.path.join(SRC_DIR, "coverage.awk")
    message:
        "Mapping: compute average assembly coverage"
    shell:
        "(date && cat {input} | awk -f {params.script} | tail -n+2 > {output} && date) &> {log}"

rule mapping_assembly_genomecov_perbase:
    input:
        os.path.join(RESULTS_DIR, "mapping/{mtype}/{rtype}/{tool}/{atype}.sr.bam")
    output:
        os.path.join(RESULTS_DIR, "mapping/{mtype}/{rtype}/{tool}/{atype}.sr.cov.perbase")
    wildcard_constraints:
        mtype="|".join(META_TYPES),
        rtype="|".join(READ_TYPES),
        tool="|".join(ASSEMBLERS),
        atype="ASSEMBLY|ASSEMBLY.FILTERED"
    threads: 1
    conda:
        os.path.join(ENV_DIR, "bedtools.yaml")
    message:
        "Mapping: compute per-base assembly coverage"
    shell:
        "bedtools genomecov -d -ibam {input} > {output}"

# gene ave. cov.
rule mapping_assembly_genomecov_pergene:
    input:
        faa=os.path.join(RESULTS_DIR, "annotation/prodigal/{rtype}/{tool}/proteins.faa"),
        cov=os.path.join(RESULTS_DIR, "mapping/{mtype}/{rtype}/{tool}/ASSEMBLY.FILTERED.sr.cov.perbase")
    output:
        os.path.join(RESULTS_DIR, "mapping/{mtype}/{rtype}/{tool}/ASSEMBLY.FILTERED.sr.cov.pergene")
    threads: 1
    run:
        from scripts.utils import parse_prodigal_faa_headers, ave_gene_cov
        genes = parse_prodigal_faa_headers(input.faa)
        genes = ave_gene_cov(input.cov, genes)
        genes.to_csv(output[0], sep="\t", header=True, index=False, index_label=False)

##################################################
# Mappability index

# flagstat: overall mapped reads, unmapped reads etc. 
# idxstats: per contig information (used this to subset contigs by length)
# unique: number of uniquely mapped reads
rule mapping_assembly_flagstat:
    input:
        os.path.join(RESULTS_DIR, "mapping/{mtype}/{rtype}/{tool}/{atype}.{rtype2}.bam")
    output:
        flagstat=os.path.join(RESULTS_DIR, "mapping/{mtype}/{rtype}/{tool}/{atype}.{rtype2}.flagstat.txt"),
        idxstats=os.path.join(RESULTS_DIR, "mapping/{mtype}/{rtype}/{tool}/{atype}.{rtype2}.idxstats.txt"),
        unique=os.path.join(  RESULTS_DIR, "mapping/{mtype}/{rtype}/{tool}/{atype}.{rtype2}.unique.txt")
    log:
        "logs/samtools.stats.{mtype}.{rtype}.{tool}.{atype}.{rtype2}.log"
    wildcard_constraints:
        mtype="|".join(META_TYPES),
        rtype="|".join(READ_TYPES),
        rtype2="|".join(READ_TYPES),
        tool="|".join(ASSEMBLERS),
        atype="ASSEMBLY|ASSEMBLY.FILTERED"
    conda:
        os.path.join(ENV_DIR, "racon.yaml")
    message:
        "Mapping: assembly coverage stats w/ samtools flagstat"
    shell:
        "(date && "
        "samtools flagstat {input} > {output.flagstat} && "
        "samtools idxstats {input} > {output.idxstats} && "
        "samtools view {input} | grep -v \"XT:A:U\" | uniq | wc -l > {output.unique} && "
        "date) &> {log}"
